import EventEmitter from 'eventemitter3';
import { splitPropertyAndMethod } from './helper';
var Container = (function () {
    function Container(modules) {
        var initialData = splitPropertyAndMethod(modules);
        this.rootState = initialData.rootState;
        this.rootReducers = initialData.rootReducers;
        this.rootEffects = initialData.rootEffects;
        this.emitter = new EventEmitter();
    }
    Container.prototype.getRootEffects = function () {
        return this.rootEffects;
    };
    Container.prototype.getRootReducers = function () {
        return this.rootReducers;
    };
    Container.prototype.getRootState = function () {
        return this.rootState;
    };
    Container.prototype.getModule = function (namespace) {
        var _this = this;
        var combineModule = {};
        var assign = function (k) {
            var _a;
            var module = {
                state: _this.rootState[k],
                reducers: _this.rootReducers[k],
                effects: _this.rootEffects[k]
            };
            Object.assign(combineModule, (_a = {}, _a[k] = module, _a));
        };
        if (Array.isArray(namespace)) {
            namespace.forEach(function (k) { return assign(k); });
        }
        else {
            assign(namespace);
        }
        return combineModule;
    };
    Container.prototype.getState = function (namespace) {
        var combineModule = this.getModule(namespace);
        return Object.keys(combineModule).reduce(function (result, key) {
            result[key] = combineModule[key].state;
            return result;
        }, {});
    };
    Container.prototype.setState = function (namespace, newState) {
        if (this.rootState[namespace]) {
            this.rootState[namespace] = newState;
        }
        this.trigger(namespace);
    };
    Container.prototype.addListener = function (namespace, listener) {
        var _this = this;
        if (Array.isArray(namespace)) {
            namespace.forEach(function (k) { return _this.emitter.on(k, listener); });
        }
        else {
            this.emitter.on(namespace, listener);
        }
    };
    Container.prototype.removeListener = function (namespace, listener) {
        var _this = this;
        if (Array.isArray(namespace)) {
            namespace.forEach(function (k) { return _this.emitter.removeListener(k, listener); });
        }
        else {
            this.emitter.removeListener(namespace, listener);
        }
    };
    Container.prototype.trigger = function (namespace) {
        this.emitter.emit(namespace);
    };
    return Container;
}());
export default Container;
